@@ -51,7 +51,11 @@ class CachedPropertyCollector(object):
 
     def __del__(self):
         if self._property_collector is not None:
-            self._property_collector.Destroy()
+            try:
+                self._property_collector.Destroy()
+            except vim.ManagedObjectNotFound:
+                # in case session ended, property collector may already be destroyed
+                pass
             self._property_collector = None
 
     def __repr__(self):
@@ -275,6 +279,7 @@ class CachedPropertyCollector(object):
         update = self._get_changes(time_in_seconds)
         return update is not None
 
+
 class HostSystemCachedPropertyCollector(CachedPropertyCollector):
     """
     Facade for fetching host attributes by using a faster traversal (e.g no need to traverse inside HostSystem)
@@ -291,7 +296,8 @@ class HostSystemCachedPropertyCollector(CachedPropertyCollector):
         select_set.append(self._create_traversal_spec(vim.Datacenter, 'hostFolder',
                           ['Folder.childEntity']))
         select_set.append(self._create_traversal_spec(vim.Folder, 'childEntity',
-                          ['Datacenter.hostFolder', 'ClusterComputeResource.host', 'ComputeResource.host']))
+                          ['Datacenter.hostFolder', 'ClusterComputeResource.host', 'ComputeResource.host',
+                           'Folder.childEntity']))
         select_set.append(self._create_traversal_spec(vim.ContainerView, 'container',
                           [select.name for select in select_set]))
         return select_set
@@ -307,11 +313,13 @@ class VirtualMachinePropertyCollector(CachedPropertyCollector):
         select_set.append(self._create_traversal_spec(vim.Datacenter, 'vmFolder',
                                                     ["Folder.childEntity"]))
         select_set.append(self._create_traversal_spec(vim.Folder, 'childEntity',
-                                                    ['Datacenter.vmFolder', "Folder.childEntity"]))
+                                                    ['Datacenter.vmFolder', "Folder.childEntity",
+                                                     'VirtualApp.vm']))
         select_set.append(self._create_traversal_spec(vim.ContainerView, 'container',
                           [select.name for select in select_set]))
         return select_set
 
+
 class TaskPropertyCollector(CachedPropertyCollector):
     def __init__(self, client, tasks, properties=["info.state"]):
         super(TaskPropertyCollector, self).__init__(client, vim.Task, properties)
